@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2016 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2016-2020 -- leonerd@leonerd.org.uk
 
 package Net::Prometheus;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.06';
+our $VERSION = '0.08';
 
 use Carp;
 
@@ -19,10 +19,14 @@ use Net::Prometheus::Counter;
 use Net::Prometheus::Summary;
 use Net::Prometheus::Histogram;
 
+use Net::Prometheus::Registry;
+
 use Net::Prometheus::ProcessCollector;
 
 use Net::Prometheus::Types qw( MetricSamples );
 
+use constant HAVE_PERLCOLLECTOR => defined eval { require Net::Prometheus::PerlCollector };
+
 =head1 NAME
 
 C<Net::Prometheus> - export monitoring metrics for F<prometheus>
@@ -82,7 +86,17 @@ Takes the following named arguments:
 
 If present and true, this instance will not load the default process collector
 from L<Net::Prometheus::ProcessCollector>. If absent or false, such a
-collector will be loaded by default. This is usually what you want.
+collector will be loaded by default.
+
+=item disable_perl_collector => BOOL
+
+If present and true, this instance will not load perl-specific collector from
+L<Net::Prometheus::PerlCollector>. If absent or false this collector is loaded
+if the module is installed and useable.
+
+These two options are provided for testing purposes, or for specific use-cases
+where such features are not required. Usually it's best just to leave these
+enabled.
 
 =back
 
@@ -94,7 +108,7 @@ sub new
    my %args = @_;
 
    my $self = bless {
-      collectors => [],
+      registry => Net::Prometheus::Registry->new,
    }, $class;
 
    if( not $args{disable_process_collector} and
@@ -102,6 +116,10 @@ sub new
       $self->register( $process_collector );
    }
 
+   if( not $args{disable_perl_collector} and HAVE_PERLCOLLECTOR ) {
+      $self->register( Net::Prometheus::PerlCollector->new );
+   }
+
    return $self;
 }
 
@@ -123,11 +141,7 @@ sub register
    my $self = shift;
    my ( $collector ) = @_;
 
-   # TODO: ban duplicate registration
-
-   push @{ $self->{collectors} }, $collector;
-
-   return $collector;
+   return $self->{registry}->register( $collector );
 }
 
 =head2 unregister
@@ -143,13 +157,7 @@ sub unregister
    my $self = shift;
    my ( $collector ) = @_;
 
-   my $found;
-   @{ $self->{collectors} } = grep {
-      not( $_ == $collector and ++$found )
-   } @{ $self->{collectors} };
-
-   $found or
-      croak "No such collector";
+   return $self->{registry}->unregister( $collector );
 }
 
 =head2 new_gauge
@@ -272,10 +280,8 @@ sub collect
 {
    my $self = shift;
 
-   my $collectors = $self->{collectors};
-
    my %samples_by_name;
-   foreach my $collector ( @{ $collectors } ) {
+   foreach my $collector ( $self->{registry}->collectors, Net::Prometheus::Registry->collectors ) {
       push @{ $samples_by_name{ $_->fullname } }, $_ for $collector->collect;
    }
 
@@ -330,7 +336,11 @@ sub render
 
       my $fullname = $metricsamples->fullname;
 
-      "# HELP $fullname " . $metricsamples->help . "\n",
+      my $help = $metricsamples->help;
+      $help =~ s/\\/\\\\/g;
+      $help =~ s/\n/\\n/g;
+
+      "# HELP $fullname $help\n",
       "# TYPE $fullname " . $metricsamples->type . "\n",
       map {
          my $sample = $_;
@@ -371,7 +381,7 @@ sub psgi_app
 
       return [
          200,
-         [ "Content-Type" => "text/plain" ],
+         [ "Content-Type" => "text/plain; version=0.0.4; charset=utf-8" ],
          [ $self->render ],
       ];
    };
@@ -428,15 +438,7 @@ information to be exported.
 
 =item *
 
-Perl-specific variable collector - arena stats?
-
-=item *
-
-Split Registry out from toplevel instance.
-
-=item *
-
-Write some actual example programs.
+Histogram/Summary 'start_timer' support
 
 =back
 
